// Buffer for the incoming data
char inData[100];
// Buffer for the parsed data chunks
char *inParse[100];

// Storage for data as string
String inString = "";

// Incoming data id
int index = 0;
// Read state of incoming data
boolean stringComplete = false;

void setup() 
{
  // Delay to facilitate start up of Xbee usually about 
  // 5 seconds. Comment out if using wired serial etc.
  
  delay(6000);
  
  // Initialise the serial port
  Serial.begin(9600);
  
  // Initialise the tank drive script  
  tankDriveSetup();
  
  // Initialise the headlight script  
  headlightSetup();
}

void loop() 
{
  if (stringComplete) 
  {
    // Parse the recieved data
    ParseSerialData();
    // Reset inString to empty
    inString = "";    
    // Reset the system for further 
    // input of data   
    stringComplete = false; 
  }  
  
  // Update the tank drive script  
  tankDriveLoop();
}

void ParseSerialData()
{
  // The data to be parsed
  char *p = inData;
  // Temp store for each data chunk
  char *str;   
  // Id ref for each chunk 
  int count = 0;
    
  // Loop through the data and seperate it into
  // chunks at each "," delimeter
  while ((str = strtok_r(p, ",", &p)) != NULL)
  { 
    // Add chunk to array  
    inParse[count] = str;
    // Increment data count
    count++;      
  }
  
  // If the data has two values then..  
  if(count == 2)
  {
    // Define value 1 as a function identifier
    char *func = inParse[0];
    // Define value 2 as a property value
    char *prop = inParse[1];
    
    // Call the relevant identified function  
    switch(*func)
    {
      case 'A': FunctionA(prop); break;
      case 'B': FunctionB(prop); break;
      case 'L': TurnOnOffLeftLight((boolean)atoi(prop)); break;
      case 'R': TurnOnOffRightLight((boolean)atoi(prop)); break;
    }    
  }
  
  if(count == 3)
  {
    // Define value 1 as a function identifier
    char *func = inParse[0];
    // Define value 2 as a property value
    char *prop = inParse[1];
    // Define value 3 as a period
    char *prod = inParse[2];
    
    // Call the relevant identified function  
    switch(*func)
    {
      case 'A': FunctionA1(prop, prod); break;

      case 'J': CalculateTankJoyDrive(prop, prod); break;
      case 'R': ApplyTankDriveRawValue(prop, prod); break;
      case 'V': ApplyTankDriveMapValue(prop, prod); break;
      case 'L': TurnOnOffBothLights(prop, prod);
    }    
  }
}

void serialEvent() 
{
  // Read while we have data
  while (Serial.available() && stringComplete == false) 
  {
    // Read a character
    char inChar = Serial.read(); 
    // Store it in char array
    inData[index] = inChar; 
    // Increment where to write next
    index++;     
    // Also add it to string storage just
    // in case, not used yet :)
    inString += inChar;
    
    // Check for termination character
    if (inChar == '\n') 
    {
      // Reset the index
      index = 0;
      // Set completion of read to true
      stringComplete = true;
    }
  }
}
